package com.suliang.common.base.activity

import android.graphics.Color
import android.os.Bundle
import androidx.viewbinding.ViewBinding
import com.suliang.common.extension.initBinding
import com.suliang.common.util.ActivityStackManager
import com.zackratos.ultimatebarx.ultimatebarx.statusBarOnly

/**
 * 基类Activity
 */
abstract class BaseActivity<VB : ViewBinding> : LifecycleLogActivity() {

    private var _binding: VB? = null

    val binding : VB
        get() = _binding!!

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        //入栈
        ActivityStackManager.addActivity(this)
        initFirst()
        initActivity(savedInstanceState)
        loadData()
    }

    /***
     * 实例化binding，
     */
    protected open fun initFirst() {
        setLayoutBefore()
        _binding = initBinding(layoutInflater)
        setContentView(binding.root)
        initStatusBar()
        setLayoutAfter()
    }

    /**
     * 状态栏初始化方法
     */
    open fun initStatusBar() {
        statusBarOnly {
            fitWindow = true
            color = Color.WHITE
            light = true
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        ActivityStackManager.removeActivity(this)
        _binding = null
    }

    /** 布局前操作 ： 空实现 */
    open fun setLayoutBefore() {

    }

    /**
     * 布局后操作：空实现
     */
    open fun setLayoutAfter() {

    }

    /**
     * onCreate中给与app中用户的具体实现
     * binding 与 ViewModel都已实现好
     * @param savedInstanceState Bundle?
     */
    abstract fun initActivity(savedInstanceState: Bundle?)

    /** onCreate()最后实现的数据加载调用 */
    abstract fun loadData()

    // 反射实现 ViewBinding
//    @SuppressWarnings("unchecked")
//    fun initBinding(inflater: LayoutInflater): VB {
//        val vbClass =
//            (javaClass.genericSuperclass as ParameterizedType).actualTypeArguments.filterIsInstance<Class<VB>>()
//        val inflate = vbClass[0].getDeclaredMethod("inflate", LayoutInflater::class.java)
//        return inflate.invoke(null, inflater) as VB
//    }

}