package com.suliang.common.util

import android.util.Log
import com.suliang.common.BuildConfig

/**
 * author suliang
 * create 2022/3/7 10:55
 * Describe:  Log日志打印框架
 */
object LogUtil {

    @JvmStatic
    private val LogEnable =
        BuildConfig.LOG_ENABLE          // 根据BuildConfig中的field LOG_ENABLE配置的值，确定是否打印

    /**
     * 创建输出内容
     * @param element StackTraceElement
     * @param message String
     * @return String
     */
    private fun createLog(element:StackTraceElement ,message : String) : String{
        return StringBuffer().apply {
            append("====${element.methodName}() : ${element.lineNumber} ==== $message ====" )
        }.toString()
    }

    fun v(message: String) {
        if (!LogEnable) return
        val stackTrace = Throwable().stackTrace[1]
        Log.v(stackTrace.fileName,createLog(stackTrace,message))
    }

    fun d(message: String) {
        if (!LogEnable) return
        val stackTrace = Throwable().stackTrace[1]
        Log.d(stackTrace.fileName,createLog(stackTrace,message))
    }

    fun e(message: String) {
        if (!LogEnable) return
        val stackTrace = Throwable().stackTrace[1]
        Log.e(stackTrace.fileName,createLog(stackTrace,message))
    }

    fun i(message: String) {
        if (!LogEnable) return
        val stackTrace = Throwable().stackTrace[1]
        Log.i(stackTrace.fileName,createLog(stackTrace,message))
    }

    fun w(message: String) {
        if (!LogEnable) return
        val stackTrace = Throwable().stackTrace[1]
        Log.w(stackTrace.fileName,createLog(stackTrace,message))
    }
}