package com.suliang.common.util.image

import android.content.Context
import android.net.Uri
import android.widget.ImageView
import androidx.annotation.DrawableRes
import java.io.File
import java.net.URL

/**
 * author suliang
 * create 2022/3/11 16:10
 * Describe:  外部调用，图片加载库
 *
 * 使用： ImageLoader.load(url).into(View)
 *
 * 顺序判断，自动限制，前面参数有值了，则后面的值无效,如果都没有值，则抛异常
 * url > file > drawableRes > uri
 *
 * https://blog.csdn.net/Coo123_/article/details/87950232
 * https://juejin.cn/post/6844903463134953479#heading-3
 * https://www.jianshu.com/p/19420b151507
 * https://blog.csdn.net/EthanCo/article/details/102587193
 */
object ImageLoader : ILoaderStrategy {

    //具体实现的策略，外部可设置
    private var mStrategy: ILoaderStrategy = GlideLoader()

    public fun setMStrategy(strategy: ILoaderStrategy) {
        this.mStrategy = mStrategy
    }


    fun loadImage(imageView: ImageView, url: String) {
        loadImage(LoaderOptions(url).apply { targetView = imageView })
    }

    fun loadImage(imageView: ImageView, file: File) {
        loadImage( LoaderOptions(file = file).apply { targetView = imageView })
    }

    fun loadImage(imageView: ImageView, drawableRes: Int) {
        loadImage(LoaderOptions(drawableResId = drawableRes).apply { targetView = imageView })
    }

    fun loadImage(imageView: ImageView, uri: Uri) {
        loadImage(LoaderOptions(uri = uri).apply { targetView = imageView })
    }

    override fun loadImage(option: LoaderOptions) {
        mStrategy.loadImage(option)
    }


    override fun clearMemoryCache() {
        mStrategy.clearMemoryCache()
    }

    override fun clearDiskCache() {
        mStrategy.clearDiskCache()
    }
}