package com.suliang.common.util.os

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Build
import android.content.res.Configuration
import android.net.Uri
import android.provider.Settings
import androidx.annotation.RequiresApi
import androidx.core.content.FileProvider
import com.suliang.common.util.AppGlobals
import com.suliang.common.util.LogUtil
import java.io.File

/**
 * author suliang
 * create 2022/3/7 15:06
 * Describe:  系统工具
 */
object SystemUtil {


    /**
     * 判断当前设备时手机还是平板
     * @param context Context  上下文
     * @return Boolean pad: true , phone: false
     */
    fun belongToPadOrPhone(context: Context): Boolean {
        val b =
            context.resources.configuration.screenLayout.and(Configuration.SCREENLAYOUT_SIZE_MASK) >= Configuration.SCREENLAYOUT_SIZE_LARGE
        LogUtil.d("是否是平板设备： $b")
        return b
    }


    /** 安装  */
    fun install(context: Context, mFilePath: String?) {
        val intent = Intent(Intent.ACTION_VIEW)
        if (VersionUtil.versionOorLater()) {
            intent.flags = Intent.FLAG_GRANT_READ_URI_PERMISSION //添加这一句表示对目标应用临时授权该Uri所代表的文件
            val apkUri = FileProvider.getUriForFile(
                context,
                "com.xuekaole.education.camera",
                File(mFilePath)
            )
            intent.setDataAndType(apkUri, "application/vnd.android.package-archive")
        } else {
            intent.setDataAndType(
                Uri.fromFile(File(mFilePath)),
                "application/vnd.android.package-archive"
            )
            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        }
        context.startActivity(intent)
    }

    @RequiresApi(api = Build.VERSION_CODES.O)
    fun isHasInstallPermissionWithO(context: Context?): Boolean {
        return context?.packageManager?.canRequestPackageInstalls() ?: false
    }

    /**
     * 开启设置安装未知来源应用权限界面
     *
     * @param context
     */
    @RequiresApi(api = Build.VERSION_CODES.O)
    fun startInstallPermissionSettingActivity(context: Context?) {
        if (context == null) {
            return
        }
        val intent = Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES)
        (context as Activity).startActivityForResult(intent, 100)
    }

    /**
     * 安装Apk   注8.0以下，8.0以上需要获取权限
     */
    fun installApk(mContext: Context, path: String) {
        val install = Intent(Intent.ACTION_VIEW)
        val file = File(path)
        if (VersionUtil.versionOorLater()) {
            val apkUri =
                FileProvider.getUriForFile(mContext, "com.xuekaole.education.download", file)
            install.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION) //添加这一句表示对目标应用临时授权该Uri所代表的文件
            install.setDataAndType(apkUri, "application/vnd.android.package-archive")
        } else {
            install.setDataAndType(Uri.fromFile(file), "application/vnd.android.package-archive")
            install.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        }
        mContext.startActivity(install)
    }


    /** 打开系统通知设置权限  */
    fun skipToNotifySetting(activity: Activity) {
        val intent = Intent()
        if (VersionUtil.versionOorLater()) {
            intent.action = "android.settings.APP_NOTIFICATION_SETTINGS"
            intent.putExtra(
                "android.provider.extra.APP_PACKAGE",
                VersionUtil.getPackageName()
            )
        } else if (VersionUtil.versionLorLater()) {
            intent.action = "android.settings.APP_NOTIFICATION_SETTINGS"
            intent.putExtra("app_package", VersionUtil.getPackageName())
            intent.putExtra("app_uid", activity.applicationInfo.uid)
        } else if (Build.VERSION.SDK_INT == Build.VERSION_CODES.KITKAT) {
            intent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
            intent.addCategory(Intent.CATEGORY_DEFAULT)
            intent.data = Uri.parse("package:" + VersionUtil.getPackageName())
        } else if (Build.VERSION.SDK_INT >= 15) {
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            intent.action = "android.settings.APPLICATION_DETAILS_SETTINGS"
            intent.data =
                Uri.fromParts("package", VersionUtil.getPackageName(), null)
        }
        activity.startActivityForResult(intent, 10000)
    }

}