package com.suliang.common.util.os

import android.os.Build
import androidx.annotation.RequiresApi
import com.suliang.common.util.AppGlobals

/**
 * author suliang
 * create 2022/3/7 16:03
 * Describe:
 */
object VersionUtil {

    /**
     * Android12 S 31 和 以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionSorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.S

    /**
     * Android11 R 30 和 以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionRorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.R

    /**
     * Android10 Q 29和 以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionQorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q

    /**
     * Android9 P 28和 以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionPorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.P

    /**
     * Android8 O 26 27 和  以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionOorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.O

    /**
     * Android7.0 N 24 和 以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionNorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.N

    /**
     * Android6.0 M 23 和 以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionMorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.M

    /**
     * Android5.0 21和 以后的版本
     * @receiver Build
     * @return Boolean
     */
    fun versionLorLater(): Boolean = Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP


    /**
     * 获取versionName
     * @return String
     */
    fun getVersionName(): String {
        return AppGlobals.application.packageManager.getPackageInfo(
            AppGlobals.application.packageName,
            0
        ).versionName
    }

    /**
     * 获取versionCode
     * @return Long
     */
    fun getVersionCode(): Long {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            AppGlobals.application.packageManager.getPackageInfo(
                AppGlobals.application.packageName,
                0
            ).longVersionCode
        } else {
            AppGlobals.application.packageManager.getPackageInfo(
                AppGlobals.application.packageName,
                0
            ).versionCode.toLong()
        }
    }

    fun getPackageName(): String {
        return AppGlobals.application.packageManager.getPackageInfo(
            AppGlobals.application.packageName,
            0
        ).packageName
    }

}