package com.suliang.common.util.thread

import java.util.concurrent.*

/**
 * author suliang
 * create 2022/3/11 9:56
 * Describe:  线程池
 * 参考url ： https://blog.csdn.net/weixin_43115440/article/details/90479752
 * newFixedThreadPool(int) : 创建固定数目线程的线程池
 * newCacheThreadPool() ： 创建一个可缓存的线程池，调用execute 将重用以前构造的线程（如果线程可用）。如果没有可用的线程，则创建一个新线程并添加到池中。终止并从缓存中移除那些已有 60 秒钟未被使用的线程。
 * newSingleThreadExecutor()：创建一个单线程化的Executor。
 * newScheduledThreadPool(int corePoolSize)：创建一个支持定时及周期性的任务执行的线程池，多数情况下可用来替代Timer类。
 *
 * FixedThreadPool 和 SingleThreadPool: 允许的请求队列长度为 Integer.MAX_VALUE，可能会堆积大量的请求，从而导致 OOM
 * CachedThreadPool 和 ScheduledThreadPool:允许的创建线程数量为 Integer.MAX_VALUE，可能会创建大量的线程，从而导致 OOM。
 *
 */
object AppExecutors {

    /* 磁盘IO线程池  和磁盘操作有关的进行使用此线程（如独写数据库，独写文件） 禁止延迟，避免等待，此线程池不用考虑同步问题 */
    val diskIO: ExecutorService = Executors.newSingleThreadExecutor()

    /** 网络线程池 网络请求、异步任务等适用此线程  不建议在此线程 sleep 或者 wait  */
    val netWork: ExecutorService = Executors.newCachedThreadPool()

    /**主线程*/
    val mainThread: Executor = MainThreadExecutor()

    /**定时任务线程池 替代Timer，执行定时任务、延时任务 * */
    val scheduledExecutor: ScheduledExecutorService =
        ScheduledThreadPoolExecutor(5, ThreadFactoryImp("executors_scheduled"), ThreadPoolExecutor.AbortPolicy())


    /*  private static ScheduledExecutorService scheduledThreadPoolExecutor(){

         1-- new ScheduledThreadPoolExecutor(5, new MyThreadFactory("scheduled"), new ThreadPoolExecutor.AbortPolicy()) ;

          2-- return new ScheduledThreadPoolExecutor(16, new ThreadFactory() {
              @Override
              public Thread newThread(Runnable r) {
                  return new Thread(r, "scheduled_executor");
              }
          }, new RejectedExecutionHandler() {
              @Override
              public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
                  CommonLog.e(TAG,"rejectedExecution: scheduled executor queue overflow");
              }
          });
      }*/

/*
    private static ExecutorService diskIoExecutor(){


        Executors.newSingleThreadExecutor(new MyThreadFactory("single")) ;
        return new ThreadPoolExecutor(1, 1, 0L, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<Runnable>(1024), new ThreadFactory() {
            @Override
            public Thread newThread(Runnable r) {
                return new Thread(r, "disk_executor");
            }
        }, new RejectedExecutionHandler() {
            @Override
            public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
                CommonLog.e(TAG,"rejectedExecution: diskIo executor queue overflow");
            }
        });
    }

    private static ExecutorService netWorkExecutor(){
        Executors.newFixedThreadPool(3, new MyThreadFactory("fixed"));

        return new ThreadPoolExecutor(3, 6, 1000, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>(6), new ThreadFactory() {
            @Override
            public Thread newThread(Runnable r) {
                return new Thread(r, "network_executor");
            }
        }, new RejectedExecutionHandler() {
            @Override
            public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
                CommonLog.e(TAG,"rejectedExecution: network executor queue overflow");
            }
        });
    }
*/
}